/*
 * A pipe pager providing access to pipe content and navigation support.
 *
 * Copyright (C) 2021 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <fsserver/pipe_accessor.h>
#include <fsserver/pipe_object_interface.h>
#include <fsserver/pipe_paging.h>
#include <fsserver/pager.h>



/* A pager abstraction for a pipe. */

class PipePager : public Pager, public PipeObject
{
protected:
    PipePaging *_paging;
    bool _writing;

    /* Helper methods. */

    virtual long next_region_for_reader(offset_t *populated_size, offset_t *size);

    virtual long next_region_for_writer(offset_t *populated_size, offset_t *size);

    virtual long pipe_error();

public:
    explicit PipePager(PipePaging *paging, bool writer);

    virtual void close();

    /* Server details. */

    int expected_items();

    ipc_server_handler_type handler();

    void *interface()
    { return static_cast<PipeObject *>(this); }

    /* Pager methods. */

    virtual long map(offset_t offset, address_t hot_spot, map_flags_t flags, l4_snd_fpage_t *region);

    /* Pipe methods. */

    virtual long closed(int *closed);

    virtual long current_region(offset_t *populated_size, offset_t *size);

    virtual long next_region(offset_t *populated_size, offset_t *size);

    /* Flushing/synchronisation. */

    virtual long flush(offset_t populated_size, offset_t *size);

    /* Notification methods. */

    virtual long subscribe(l4_cap_idx_t notifier, notify_flags_t flags);

    virtual long unsubscribe();
};

// vim: tabstop=4 expandtab shiftwidth=4
